;-------------------------------------------------------------------------------
;PLAINJUMP (C) 1997 BY ANDREAS ESS
;                      TUFERS 11
;                      A-6811 GOEFIS
;                      AUSTRIA/EUROPE
;                      ess.andreas@computerhaus.at
;                      http://andi.ganymed.org
;
;This is the Usgard version of Plainjump, nearly the same, but 300 bytes smaller
;through the use of Usgard functions + another ? bytes by clearing out stuff.
;Have fun with it,
;       Andreas
;-------------------------------------------------------------------------------

#INCLUDE "USGARD.H"             ;use the Usgard header file!
#INCLUDE "ZCPLIB.H"             ;ZCP library
#INCLUDE "USGDLIB.H"

;-------------------------------------------------------------------------------
;Variables in text-memory:
;-------------------------------------------------------------------------------
LEVELY =  TEXT_MEM      ;position in level
SCROLLY = TEXT_MEM+1    ;position in level-y
LEVELPT = TEXT_MEM+2    ;pointer to level data
;FREE WORD
PLAYERPT = TEXT_MEM+6   ;pointer to player
JUMPFLAG = TEXT_MEM+8   ;0 = no jump; 1 = up; 2 = down
PLAYERPT1 = TEXT_MEM+9  ;pointer to black-player image
ANICOUNT = TEXT_MEM+11  ;animation counter
SPEED = TEXT_MEM+12     ;speed of ball
LIFE = TEXT_MEM+14      ;lifes of player
SCORE = TEXT_MEM+15     ;contains player's score
ORIGLVL = TEXT_MEM+17   ;pointer to start of level
BALLY = TEXT_MEM+19     ;only needed for savegame
BALLX = TEXT_MEM+20
Data = TEXT_MEM+21      ;needed for ZCP
StringPlace = TEXT_MEM+22
ZeroByte = TEXT_MEM+27
HiScoreTemp = TEXT_MEM+28
Count = TEXT_MEM+30
TempByte = TEXT_MEM+31
MASK = TEXT_MEM+32
ANISCROLLC = TEXT_MEM+33
KEYPRESS = TEXT_MEM+34

BackgroundTemp = TEXT_MEM2
;-------------------------------------------------------------------------------
BITMAPMEM = $8641 ;this points to the graph mem

.org    0
.db     "PlaneJump v1.0 by AE",0
.db     0


;-------------------------------------------------------------------------------
;Program start!
;-------------------------------------------------------------------------------
Title:
	ld	hl, USG_BITS
	set	4, (hl)			;Ignore if unable to find a library

	ld      a, 4                    ;Set page 4 for FIND_PIXEL
	out     (5), a
	call    CLEARLCD                ;In Usgard, you may use either
					;a normal call or a ROM_CALL!
	ld      c, ZCPLIB
	call    LIB_CHECK
	jr      c, StartWrite
	ld      hl, &TitlePicture       ;Automatic addition of PROGRAM_ADDR
					;using a & in front of a label!
	LIB_CALL_(ZCPLIB,ZCP)           ;use libraries to make your progs more
					;efficient and smaller!
Wait:
	call    GET_KEY                 ;A = key
	and     a                       ;no key pressed?
	jr      z, Wait

StartWrite:
	ld      hl, $0002               ;first byte=XPos,second byte=YPos
	ld      (CURSOR_ROW), hl        ;(this is line 2)
	ld      hl, &Instr              ;Use the & operator together with RELOC.EXE
	call    D_ZT_STR                ;by Jimmy to add PROGRAM_ADDR automatically
	ld      hl, &HiScore            ;(for stuff inside the program)
	call    LD_HL_MHL               ;load the contents of the address HiScore
					;into HL
	call    D_HL_DECI               ;display HL as number.

	ld	hl, $300A
	ld	(CURSOR_X), hl
	ld	hl, &EMail
	call	D_ZM_STR

	ld      (iy+8), $8C             ;SWITCH ON APD!!!

WaitForKey:
	call    GET_KEY
	and     a                       ;is A = 0? (faster than CP 0)
	jr      z, WaitForKey           ;Yes, A = 0 -> no key pressed
					; -> jump to WaitForKey
	call    $115                    ;Key has been pressed, set APD to 5 minutes again
	ld      (iy+8), $84             ;Switch APD off.

	cp      K_EXIT                  ;is A = K_EXIT(keycode for EXIT)
	jp      z, &ExitGame            ;yes, FAASSSTTTT jump to ExitGame!
	cp      K_F3                    ;A = F3?
	jp      z, &LevelEditor         ;yo, go to the level editor
	cp      K_F2                    ;A = F2?
	jr      z, LoadExternGame       ;yup, load a user level
	cp      K_F1                    ;A = F1?
	jr      z, LoadGame             ;yeah, load game
	cp      K_SECOND
	jr      z, RegularGame
	cp      K_ENTER
	jr      z, RegularGame
	jr      WaitForKey
;Wait until F1 released
LoadGame:
	xor     a
	ld      (LAST_KEY), a
	call    CLEARLCD
	ld      hl, &LevelSaved
	ld      a, (hl)
	and     a
	jr      z, RegularGame
;Copy saved game(20 bytes) to TEXT_MEM
	ld      bc, 21
	inc     hl
	ld      de, LEVELY
	ldir
	call    &ClearDBuf              ;FAASSSTTTT calls!
	call    &DrawScreen

	ld      bc, (BALLY)
	xor     a
	ld      (SCROLLY), a
	call    &ScanBk

WaitForNoKey:
	call    GET_KEY
	and     a
	jr      nz, WaitForNoKey
	jp      &GameLoop

LoadExternGame:
	ld      de, (PROGRAM_ADDR)      ;only use PROGRAM_ADDR if you need
	ld      hl, Level               ;more than twice.
	add     hl, de
	ld      (LEVELPT), hl
	ld      (ORIGLVL), hl
	jr      BuiltInLevels
RegularGame:
 ;Do some initialisation
	ld      de, (PROGRAM_ADDR)
	ld      hl, Level1
	add     hl, de
	ld      (LEVELPT), hl
	ld      (ORIGLVL), hl
BuiltInLevels:
	ld      hl, WhiteBall
	add     hl, de
	ld      (PLAYERPT), hl
	ld      hl, BlackBall1
	add     hl, de
	ld      (PLAYERPT1), hl
	call    CLEARLCD
	xor     a
	ld      (SCROLLY), a
	ld      (LEVELY), a
	ld      (JUMPFLAG), a
	ld      (ANICOUNT), a
	ld      (SPEED), a
	ld      h, a
	ld      l, a
	ld      (SCORE), hl
	ld      a, 6
	ld      (LIFE), a

;Draw screen first time:
StartGameLoop:
	call    &ClearDBuf
	call    &DrawScreen
	ld      hl, (LEVELPT)
	inc     hl
	ld      (LEVELPT), hl
	ld      b, 52
	ld      c, 38
	call    &ScanBk

;-------------------------------------------------------------------------------
;The game loop:
;-------------------------------------------------------------------------------
GameLoop:
	call    &DrawBk
	ld      a, c
	cp      38
	jr      nz, TestKeys
	push    bc
	ld      a, b
	add     a, 12
	ld      b, a
	ld      a, c
	add     a, 20
	ld      c, a
	call    &TestPixel
	pop     bc
	jp      z, &LostLife
TestKeys:
	call    OTH_ARROW               ;OTH_ARROW returns most used multikeys:
					;bit 0: down
	and     a                       ;bit 1: left
					;bit 2: right
	jp      z, &Redraw              ;bit 3: up
TestLeftKey:                            ;bit 4: F1
	ld      d, a                    ;bit 5: 2nd
	ld      a, (SPEED)              ;bit 6: exit
	ld      e, a                    ;bit 7: more
	bit     1, d                    ;It's best to load the result into an
	jr      nz, TestRightKey        ;unused register an perform bit ?, reg
	ld      a, b
	cp      2
	jr      c, TestRightKey
	dec     b
	ld      a, e
	cp      3
	jr      nz, TestRightKey
	dec     b
TestRightKey:
	bit     2, d
	jr      nz, TestUpKey
	ld      a, b
	cp      103
	jr      nc, TestUpKey
	inc     b
	ld      a, e
	cp      3
	jr      nz, TestUpKey
	inc     b
TestUpKey:
	bit     3, d
	jr      nz, TestDownKey
	ld      a, e
	cp      3
	jr      nc, TestDownKey
	inc     a
	ld      (SPEED), a
TestDownKey:
	bit     0, d
	jr      nz, Test2ndKey
	ld      a, e
	and     a
	jr      z, Test2ndKey
	dec     a
	ld      (SPEED), a
Test2ndKey:     ;JUMP?
	bit     5, d
	jr      nz, TestExitKey
	ld      a, (JUMPFLAG)
	and     a
	jr      nz, TestExitKey
	inc     a
	ld      (JUMPFLAG), a

TestExitKey:
	bit     6, d
	jp      z, &ExitGame
TestPauseKey:
	bit     7, d
	jp      z, &Pause
TestF1Key:
	bit     4, d
	jp      z, &LSaveGame

Redraw:
	ld      a, (JUMPFLAG)
	and     a
	jr      z, ReallyRedraw
	dec     a
	jr      nz, FlyDown
	ld      a, c
	cp      20
	jr      z, SetFlyDown
	dec     c
	jr      ReallyRedraw
SetFlyDown:
	ld      a, 2
	ld      (JUMPFLAG), a
FlyDown:
	ld      a, c
	cp      38
	jr      nz, FlyDown1
	xor     a
	ld      (JUMPFLAG), a
	jr      ReallyRedraw
FlyDown1:
	inc     c
ReallyRedraw:
	ld      a, (SPEED)
	and     a
	jr      z, RedrawBall
	push    bc
	ld      b, a
DoScrolling:
	call    &DoScroll
	djnz    DoScrolling
	pop     bc

RedrawBall:
	call    &ScanBk
	ld      de, (PLAYERPT)
	call    &WhiteSprite
	ld      de, (PLAYERPT1)
	call    &BlackSprite

	ld      a, (ANISCROLLC)
	ld      hl, SPEED
	add     a, (hl)
	ld      (ANISCROLLC), a
	cp      6
	jr      c, GoOnMainLoop

	xor     a
	ld      (ANISCROLLC), a
	ld      a, (ANICOUNT)
	inc     a
	ld      (ANICOUNT), a
	cp      3
	jr      nz, NextFrame
	xor     a
	ld      (ANICOUNT), a
	ld      hl, &BlackBall1
	ld      (PLAYERPT1), hl
	jr      GoOnMainLoop
NextFrame:
	ld      hl, (PLAYERPT1)
	ld      de, 72
	add     hl, de
	ld      (PLAYERPT1), hl

GoOnMainLoop:
 ;copy GRAPH-MEM to LCD-MEM
	ld      de, VIDEO_MEM
	ld      hl, BITMAPMEM
	push    bc
	ld      bc, 1024
	ldir
	pop     bc
	jp      &GameLoop

Pause:
	call    GET_KEY
	and     a
	jr      z, Pause
	jp      &GameLoop

LostLife:
	xor     a
	ld      (SPEED), a
NoFade:
	ld      a, (LIFE)
	dec     a
	jr      z, GameOver
	call    CLEARLCD
	ld      (LIFE), a
	ld      hl, (LEVELPT)
	xor     a
	ld      (SCROLLY), a
FindStone:
	dec     hl
	dec     hl
	ld      (LEVELPT), hl
	ld      a, (LEVELY)
	dec     a
	dec     a
	ld      (LEVELY), a
	jp      z, &StartGameLoop
	ld      c, $C0
	ld      b, 1
FindStone1:
	ld      a, (hl)
	and     c
	jr      nz, FoundDone
	srl     c
	srl     c
	jr      z, FindStone
	ld      a, b
	add     a, 32
	ld      b, a
	jr      FindStone1
FoundDone:
	ld      c, 38
	push    bc
	call    &DrawScreen
	pop     bc
	call    &ScanBk
	jp      &GameLoop

GameOver:
	ld      a, (CONTRAST)
	out     (2), a
	call    CLEARLCD
	call    &HiScoreHand
	jp      &Title

LSaveGame:
	ld      (BALLY), bc
	ld      hl, &LevelSaved
	ld      (hl), 1
	inc     hl
	ld      de, LEVELY
	ex      de, hl
	ld      bc, 21
	ldir
	ld      hl, USG_BITS
	set     1, (hl)              ;Tell Usgard to warp out...
	ret

ExitGame:
	ld      a, (CONTRAST)
	out     (2), a
	call    &ClearDBuf
	ret

;-------------------------------------------------------------------------------
;ClearDBuf: clears graphmem which is used as double buffer
;-------------------------------------------------------------------------------
ClearDBuf:
	ld      hl, BITMAPMEM
	ld      bc, 1023
	call    OTH_CLEAR       ;OTH_CLEAR clears BC+1 bytes at HL
	ret

;-------------------------------------------------------------------------------
;WhiteLine: draws a horizontal, white line from (B,D) to (C,D)
;-------------------------------------------------------------------------------
WhiteLine:
 push hl
 push de
 ld   h, 0
 ld   l, d
 add  hl, hl    ;multiply a by 2
 add  hl, hl    ;            * 4
 add  hl, hl    ;            * 8
 add  hl, hl    ;            * 16
 push bc
 srl  b         ;divide b by 8
 srl  b
 srl  b
 ld   d, 0
 ld   e, b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 pop  bc
 ;HL now contains video-offset
 ld   a, b      ;load real x1-pos into a
 ld   d, a      ;d=real x1-pos is needed later...
 and  7         ;and make the mask
 ld   b, 128    ;b is the video-mask
 jr   z, WLoop  ;mod(a,7) = 0->no mask-shift needed
 ld   e, 128
 ld   b, a      ;times to shift
ShiftWLoop:
 srl  e
 djnz ShiftWLoop
 ld   b, e
WLoop:          ;ready-go!
 ld   a, b                 ;erase point
 cpl
 and  (hl)
 ld   (hl), a   ;and save it back to vid-ram

 dec  c         ;decrement X2 if X2 = X1 -> end
 ld   a, c
 cp   d         ;now know for what d was needed?
 jr   z, EndWhite

 rrc  b         ;shift mask
 jr   nc, WLoop
 inc  hl
 jr   WLoop
EndWhite:
 pop  de
 pop  hl
 ret

;-------------------------------------------------------------------------------
;BlackLine: draws a horizontal, black line from (B,D) to (C,D)
;-------------------------------------------------------------------------------
BlackLine:
 push hl
 push de
 ld   h, 0
 ld   l, d
 add  hl, hl    ;multiply a by 2
 add  hl, hl    ;            * 4
 add  hl, hl    ;            * 8
 add  hl, hl    ;            * 16
 push bc
 srl  b         ;divide b by 8
 srl  b
 srl  b
 ld   d, 0
 ld   e, b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 pop  bc
 ;HL now contains video-offset
 ld   a, b      ;load real x1-pos into a
 ld   d, a      ;d=real x1-pos is needed later...
 and  7         ;and make the mask
 ld   b, 128    ;b is the video-mask
 jr   z, BLoop  ;mod(a,7) = 0->no mask-shift needed
 ld   e, 128
 ld   b, a      ;times to shift
ShiftBLoop:
 srl  e
 djnz ShiftBLoop
 ld   b, e
BLoop:          ;ready-go!
 ld   a, (hl)   ;load byte from vid-ram
 or   b         ;set it to black
 ld   (hl), a   ;and save it back to vid-ram

 dec  c         ;decrement X2 if X2 = X1 -> end
 ld   a, c
 cp   d         ;now know for what d was needed?
 jr   z, EndBlack

 rrc  b         ;shift mask
 jr   nc, BLoop
 inc  hl
 jr   BLoop
EndBlack:
 pop  de
 pop  hl
 ret

;-------------------------------------------------------------------------------
;DrawLine: this draws a line at y-pos D, accoriding to bits in E
;-------------------------------------------------------------------------------
DrawLine:
 push bc
 ld     hl, &StartLine1
 ld   b,  0
 ld   a,  63
 sub  d
 srl  a
 ld   c,  a
 add  hl, bc

 ;HL now contains pointer to StartLine1

 ld   b, (hl)  ;\   set
 push de
 ld   de, 32
 add  hl, de   ;  > &
 pop  de
 ld   c, (hl)  ; /  X2

 ld   a, e
 srl  a
 and  $C0
 jr   z, WhiteLn1
 call   &BlackLine
 jr   Line2
WhiteLn1:
 call   &WhiteLine

Line2:
 ld   b, (hl)  ;\   set
 push de
 ld   de, 32
 add  hl, de   ;  > &
 pop  de
 ld   c, (hl)  ; /  X2

 ld   a, e
 and  $30
 jr   z, WhiteLn2
 call   &BlackLine
 jr   Line3
WhiteLn2:
 call   &WhiteLine

Line3:
 ld   b, (hl)  ;\   set
 push de
 ld   de, 32
 add  hl, de   ;  > &
 pop  de
 ld   c, (hl)  ; /  X2

 ld   a, e
 and  $0C
 jr   z, WhiteLn3
 call   &BlackLine
 jr   Line4
WhiteLn3:
 call   &WhiteLine

Line4:
 ld   b, (hl)  ;\   set
 push de
 ld   de, 32
 add  hl, de   ;  > &
 pop  de
 ld   c, (hl)  ; /  X2
 ld   a, e
 and  $03
 jr   z, WhiteLn4
 call   &BlackLine
 jr   EndLine4
WhiteLn4:
 call   &WhiteLine

EndLine4:
 pop  bc
 ret

;-------------------------------------------------------------------------------
;Drawscreen
;-------------------------------------------------------------------------------
DrawScreen:
 ld   hl, (LEVELPT)
 ld   e, (hl)
 ld   b, 32
DrawScrLoop:
 ld   a, b
 add  a, 32
 ld   d, a
 call   &DrawLine
 djnz DrawScrLoop
 ld   hl, (LEVELPT)
 inc  hl
 ld   e, (hl)
 ld   b, 32
DrawScrLoop1:
 ld   d, b
 call   &DrawLine
 djnz DrawScrLoop1
 ret

;-------------------------------------------------------------------------------
;DoScroll: handles scrolling
;-------------------------------------------------------------------------------
DoScroll:
 ld   a, (SCROLLY)   ;increment SCROLLY
 inc  a
 ld   d, a
 ld   (SCROLLY), a   ;save it
 cp   32             ;SCROLLY = 32 ?
 jr   nz, LoadHLScr
 xor  a
 ld   d, a
 ld   (SCROLLY), a
 ld   hl, $0000
 ld   ($8333), hl
 ld   hl, (SCORE)
 call DM_HL_DECI	;display HL menu style
 push bc
 ld   b, 6
 ld   de, BITMAPMEM
 ld   hl, VIDEO_MEM
CopyVid2Grf:
 push bc
 ldi
 ldi
 ldi
 ld   bc, 13
 add  hl, bc
 ex   de, hl
 add  hl, bc
 ex   de, hl
 pop  bc
 djnz CopyVid2Grf
 pop  bc
 ld   a, (LEVELY)    ;increment LEVELY
 inc  a
 ld   (LEVELY), a
 cp   254
 jr   nz, GoOnScroll
 xor  a
 ld   (LEVELY), a
 ld   hl, (ORIGLVL)
 ld   (LEVELPT), hl
GoOnScroll:
 ld   hl, (SCORE)
 ld   de, $0002
 add  hl, de
 ld   (SCORE), hl
 ld   hl, (LEVELPT)  ;and increment LEVELPT
 inc  hl
 ld   (LEVELPT), hl
LoadHLScr:
 ld   hl, (LEVELPT)  ;get Levelinformation from LEVELPT
 inc  hl
 ld   e, (hl)
 call   &DrawLine
 ld   a, (SCROLLY)
 add  a, 32
 ld   d, a
 ld   hl, (LEVELPT)
 ld   e, (hl)
 call   &DrawLine
 ret

;-------------------------------------------------------------------------------
;ScanBk: stores background from(B,C)-(B+24,C+24) in BKPT
;-------------------------------------------------------------------------------
ScanBk:
 push bc
 ld   h, 0
 ld   l, c
 add  hl, hl    ;multiply a by 2
 add  hl, hl    ;            * 4
 add  hl, hl    ;            * 8
 add  hl, hl    ;            * 16
 srl  b         ;divide b by 8
 srl  b
 srl  b
 ld   d, 0
 ld   e, b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 ld   de, BackgroundTemp
 ;HL contains Video-Offset
 ;DE contains BkPt
 ;Now we are ready...
 ld   b, 23       ;23 lines to scan
ScanLoop:
 push bc
 ld   bc, 4
 ldir             ;move 4 bytes from hl to de
 pop  bc
 push de          ;next line in vid-ram
 ld   de, 12
 add  hl, de
 pop  de
 djnz ScanLoop
 pop  bc
 ret

;-------------------------------------------------------------------------------
;DrawBk: draws background from(B,C)-(B+24,C+24) in BKPT
;-------------------------------------------------------------------------------
DrawBk:
 push bc
 ld   h, 0
 ld   l, c
 add  hl, hl    ;multiply a by 2
 add  hl, hl    ;            * 4
 add  hl, hl    ;            * 8
 add  hl, hl    ;            * 16
 srl  b         ;divide b by 8
 srl  b
 srl  b
 ld   d, 0
 ld   e, b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 ld   de, BackgroundTemp
 ex   de, hl
 ;DE contains Video-Offset
 ;HL contains BkPt
 ;Now we are ready...
 ld   b, 23       ;23 lines to draw
PutLoop:
 push bc
 ld   bc, 4
 ldir              ;move 4 bytes from hl to de
 pop  bc
 push hl          ;next line in vid-ram
 ld   hl, 12
 add  hl, de
 ex   de, hl
 pop  hl
 djnz PutLoop
 pop  bc
 ret

;-----------------------------------------------------------------------------
;WhiteSprite: draws each white element of a 24*24 sprite
;-----------------------------------------------------------------------------
WhiteSprite:
 push bc
 ld   a, 0
 ld   (TempByte), a ;Line
 ld   (Count), a    ;Column
 ld   h, 0
 ld   l, c
 add  hl, hl ;*2
 add  hl, hl ;*4
 add  hl, hl ;*8
 add  hl, hl ;*16
 push de
 push bc
 srl  b
 srl  b
 srl  b
 ld   d,  0
 ld   e,  b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 pop  bc
 pop  de
 ld   a, b
 and  7
 ld   c, 128
 ld   b, c
 jr   z, WSLoop
 ld   b, a
WSShift:
 srl  c
 djnz WSShift
 ld   b, 128
WSLoop:
 ld   a, (de)
 and  b
 jr   nz, WSLoop1
 ld   a, c
 cpl
 and  (hl)
 ld   (hl), a
WSLoop1:
 rrc  c
 jr   nc, WSLoop2
 inc  hl
WSLoop2:
 rrc  b
 jr   nc, WSLoop
WSNextLine:
 inc  de
 ld   a, (Count)
 inc  a
 ld   (Count), a
 cp   3
 jr   nz, WSLoop
 xor  a
 ld   (Count), a
 push de
 ld   de, 13
 add  hl, de
 pop  de
 ld   a, (TempByte)
 inc  a
 ld   (TempByte), a
 cp   23
 jr   nz, WSLoop
WSEndBMP:
 pop  bc
 ret

;-----------------------------------------------------------------------------
;BlackSprite: draws each black element of a 24*24 sprite
;-----------------------------------------------------------------------------
BlackSprite:
 push bc
 ld   a, 0
 ld   (TempByte), a ;Line
 ld   (Count), a    ;Column
 ld   h, 0
 ld   l, c
 add  hl, hl ;*2
 add  hl, hl ;*4
 add  hl, hl ;*8
 add  hl, hl ;*16
 push de
 push bc
 srl  b
 srl  b
 srl  b
 ld   d,  0
 ld   e,  b
 add  hl, de
 ld   de, BITMAPMEM
 add  hl, de
 pop  bc
 pop  de
 ld   a, b
 and  7
 ld   c, 128
 ld   b, 128
 jr   z, BSLoop
 ld   b, a
BSShift:
 srl  c
 djnz BSShift
 ld   b, 128
BSLoop:
 ld   a, (de)
 and  b
 jr   z, BSLoop1
 ld   a, (hl)
 or   c
 ld   (hl), a
BSLoop1:
 rrc  c
 jr   nc, BSLoop2
 inc  hl
BSLoop2:
 rrc  b
 jr   nc, BSLoop
BSNextLine:
 inc  de
 ld   a, (Count)
 inc  a
 ld   (Count), a
 cp   3
 jr   nz, BSLoop
 xor  a
 ld   (Count), a
 push de
 ld   de, 13
 add  hl, de
 pop  de
 ld   a, (TempByte)
 inc  a
 ld   (TempByte), a
 cp   23
 jr   nz, BSLoop
BSEndBMP:
 pop  bc
 ret

;-----------------------------------------------------------------------------
;Delay: waits
;-----------------------------------------------------------------------------
Delay:
 push af
 push bc
 ld bc, $1000
DelayLoop:
 dec bc
 ld a, b
 or c
 jr nz, DelayLoop
 pop bc
 pop af
 ret

;-------------------------------------------------------------------------------
;Testpixel : returns A = 0 if pixel white
;-------------------------------------------------------------------------------
TestPixel:
 push bc
 ld     a, 63
 sub    c
 ld     c, a
 call   FIND_PIXEL
 ld   de, BITMAPMEM
 add  hl, de
 and  (hl)
 pop  bc
 ret

;-----------------------------------------------------------------------------
;Handle Hi-score
;-----------------------------------------------------------------------------
HiScoreHand:
 call   CLEARLCD
 ld     hl, &HiScore
 call   LD_HL_MHL
 ld     de, (SCORE)
 ld     (HiScoreTemp), de
;DE contains hiscore, HL actual score
 call   CP_HL_DE
 ret    nc
 ld     hl, &HiScore
 ex     de, hl
 ld     hl, HiScoreTemp
 ld     c, 2
 ldi
 ldi
 ld     hl, $0300
 ld     ($800C), hl
 ld     hl, &HiScr
 call   D_ZT_STR
 ld     hl, $0A1F
 ld     ($8333), hl
 ld     hl, &EnterInit
 call   D_ZM_STR
 ld     hl, $0803
 ld     ($800C), hl
 ld     ix, &HiScrName
 ld     c, 3
 ld	a, 00010000b
 LIB_CALL_(USGDLIB,INPST)
 ret

;-------------------------------------------------------------------------------
;Built-in Level editor:
;-------------------------------------------------------------------------------
LevelEditor:
 ld     hl, &Level
 ld  (LEVELPT), hl
 xor a
 ld  (LEVELY), a
 call   &EDrwScreen
 ld  a, $C0
 ld  (MASK), a
 ld  b, 0
 ld  c, 60
KeyLoop:
 call   &PointChg
WaitKey:
 call GET_KEY
 and  a
 jr   z, WaitKey
 push af
 call   &PointChg
 pop  af
 cp   $01
 jr   z, MoveDown
 cp   $02
 jr   z, MoveLeft
 cp   $03
 jr   z, MoveRight
 cp   $04
 jr   z, MoveUp
 cp   $36
 jp     z, &ChangeStone
 cp   $37
 jp     z, &ExitEditor
 cp   $0F
 jp     z, &ClearLevel
 jr   KeyLoop
MoveDown:
 ld   a, (LEVELY)
 and  a
 jr   z, KeyLoop
 dec  a
 ld   (LEVELY), a
 inc  c
 inc  c
 ld   a, c
 cp   62
 jr   nz, KeyLoop
 ld   hl, (LEVELPT)
 dec  hl
 ld   (LEVELPT), hl
 ld   c, 60
 call   &EDrwScreen
 jr   KeyLoop
MoveLeft:
 ld   a, b
 and  a
 jr   z, KeyLoop
 dec  b
 dec  b
 ld   a, (MASK)
 add  a, a
 add  a, a
 ld   (MASK), a
 jr   KeyLoop
MoveRight:
 ld   a, b
 cp   6
 jr   z, KeyLoop
 inc  b
 inc  b
 ld   a, (MASK)
 srl  a
 srl  a
 ld   (MASK), a
 jr   KeyLoop
MoveUp:
 ld   a, (LEVELY)
 cp   253
 jp     z, &KeyLoop
 inc  a
 ld   (LEVELY), a
 dec  c
 dec  c
 ld   a, c
 and  a
 jp     nz, &KeyLoop
 ld   hl, (LEVELPT)
 inc  hl
 ld   (LEVELPT), hl
 ld   c, 2
 call   &EDrwScreen
 jp     &KeyLoop
ChangeStone:
 ld   hl, (LEVELPT)
 ld   a, 60
 sub  c
 ld   e, a
 srl  e
 ld   d, 0
 add  hl, de
 ld   a, (MASK)
 xor  (hl)
 ld   (hl), a
 call   &EDrwScreen
 jp     &KeyLoop

ClearLevel:
 ld   hl, (LEVELPT)
 ld   b, 255
 ld   a, 0
ClearLL:
 ld   (hl), a
 inc  hl
 djnz ClearLL
 call   &EDrwScreen
 jp     &KeyLoop

ExitEditor:
 ld   hl, (LEVELPT)
 ld   de, (LEVELPT)
 ld   bc, 254
 add  hl, bc
 ex   de, hl
 ldi
 ldi
 jp     &Title

;-----------------------------------------------------------------------------
;EDrwScreen: draws screen
;-----------------------------------------------------------------------------
EDrwScreen:
 push bc
 call   CLEARLCD
 ld  hl, (LEVELPT)
 ld  b, 30
DrawLoop:
 ld   a, b
 add  a, a
 ld   c, a
 push bc
 bit 6, (hl)
 ld  b, 0
 call   nz, &DrawBox
 bit 4, (hl)
 ld  b, 2
 call   nz, &DrawBox
 bit 2, (hl)
 ld  b, 4
 call   nz, &DrawBox
 bit 0, (hl)
 ld  b, 6
 call   nz, &DrawBox
 pop bc
 inc  hl
 djnz DrawLoop
 pop bc
 ld  l, 10
 ld  h, 1
 ld  de, Arrow
 call  &DisplayText
 ld  h, 8
 ld  de, Second
 call   &DisplayText
 ld  h, 15
 ld  de, Exit
 call   &DisplayText
 ret

;------------------------------------------------------------------------------
;DisplayText: displays the text in DE at position(H,L) in menustyle
;------------------------------------------------------------------------------
DisplayText:
 push hl
 ld   ($8333), hl
 ld   hl, (PROGRAM_ADDR)
 add  hl, de
 call   D_ZM_STR
 pop  hl
 ret

;-----------------------------------------------------------------------------
;DrawBox
;-----------------------------------------------------------------------------
DrawBox:
 push bc
 call   &PointOn
 inc b
 call   &PointOn
 inc c
 call   &PointOn
 dec b
 call   &PointOn
 pop bc
 ret

;-----------------------------------------------------------------------------
;PointOn: draws point at b=x, c=y
;-----------------------------------------------------------------------------
PointOn:
 push hl
 push bc
 ld   a, 63
 sub  c
 ld   c, a
 call   FIND_PIXEL  ;HL=byte offset in video buffer, A=2^(bit to change)
 ld de, $FC00
 add hl, de
 or (hl)
 ld (hl),a
 pop bc
 pop hl
 ret

;-----------------------------------------------------------------------------
;PointChg: changes point at b=x, c=y
;-----------------------------------------------------------------------------
PointChg:
 push bc
 push hl
 ld   a, 63
 sub  c
 ld   c, a
 call   FIND_PIXEL
 ld   de,$FC00
 add  hl,de
 xor  (hl)
 ld   (hl),a
 pop  hl
 pop  bc
 ret

;-------------------------------------------------------------------------------
;Some important variables:
;-------------------------------------------------------------------------------
Arrow:     .db "ARROWS TO MOVE,",0
Second:    .db "2nd TO CHANGE",0
Exit:      .db "EXIT TO SAVE&RETURN",0
		    ;012345678901234567890
Instr:          .db "    [F1]  Restore    "
		.db "      [F2] User      "
		.db "     [F3] Editor     "
		.db "HiScore by "
HiScrName: .db "AND",0
HiScore:   .dw 10                       ;This is standard hi-score
HiScr:     .db "NEW HIGH SCORE!",0
EnterInit: .db "ENTER YOUR INITIALS:",0
	       ;123456789012345678901
EMail:     .db "ess.andreas@computerhaus.at",0

;-------------------------------------------------------------------------------
;This array contains start&end-line indexes(?)
;-------------------------------------------------------------------------------
StartLine1:
 .db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27
 .db 28,29,30,31
StartLine2:
 .db 31,31,32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39,40,40,41,41,42,42
 .db 43,43,44,44,45,45,46,46
StartLine3:
 .db 63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63,63
 .db 63,63,63,63,63,63,63,63
StartLine4:
 .db 96,96,95,95,94,94,93,93,92,92,91,91,90,90,89,89,88,88,87,87,86,86,85,85
 .db 84,84,83,83,82,82,81,81
StartLine5:
 .db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112,111,110
 .db 109,108,107,106,105,104,103,102,101,100,99,98,97,96

;-------------------------------------------------------------------------------
;This is the built-in level:
;-------------------------------------------------------------------------------
Level1:
 .db $FF,$3C,$FF,$3C,$0C,$0C,$0F,$03,$03,$03,$C0,$C0,$C0,$FF,$FF,$00 ;16
 .db $FF,$FF,$00,$3C,$3C,$00,$03,$0C,$30,$C0,$FF,$3C,$C3,$C3,$03,$FF ;32
 .db $33,$33,$03,$30,$03,$30,$03,$C0,$FF,$03,$03,$C0,$0C,$C3,$CF,$CF ;48
 .db $C0,$C0,$FF,$00,$0C,$00,$30,$00,$03,$00,$C0,$F3,$F3,$F3,$F0,$C0 ;72
 .db $C0,$F0,$30,$3C,$0C,$0F,$03,$03,$03,$F0,$F0,$00,$FF,$3C,$FF,$C0 ;80
 .db $00,$30,$FF,$FC,$F0,$C0,$03,$C0,$03,$FF,$FF,$F3,$FF,$CF,$FF,$C3 ;96
 .db $3C,$00,$C3,$00,$03,$00,$00,$FF,$FF,$03,$0C,$30,$C0,$30,$C0,$03 ;112
 .db $FF,$00,$00,$3C,$00,$03,$00,$00,$FF,$00,$00,$FF,$FF,$F3,$CF,$F3 ;128
 .db $00,$00,$C0,$00,$00,$00,$FF,$0C,$0C,$0F,$03,$03,$0F,$0C,$0F,$30 ;144
 .db $F0,$C0,$C0,$00,$FF,$00,$00,$03,$03,$03,$03,$03,$C0,$C0,$00,$00 ;160
 .db $FF,$F3,$FC,$F3,$CF,$3F,$3F,$00,$00,$FF,$00,$00,$FF,$00,$00,$FF ;176
 .db $C3,$3C,$C3,$F3,$FC,$FF,$00,$00,$00,$03,$FF,$C0,$F0,$03,$3C,$0C ;192
 .db $0F,$03,$03,$00,$00,$C0,$FF,$FF,$F3,$CF,$CF,$F3,$FF,$03,$0C,$03 ;208
 .db $00,$FF,$3C,$FF,$00,$FF,$33,$CC,$CC,$33,$3C,$00,$FF,$00,$00,$00 ;224
 .db $FF,$FF,$03,$C0,$30,$0C,$30,$0C,$FF,$00,$FF,$F3,$CF,$3F,$00,$FF ;240
 .db $FF,$00,$FF,$00,$3C,$00,$3C,$00,$3C,$00,$30,$00,$30,$00,$00,$00,$FF ;256

;-------------------------------------------------------------------------------
;This is the edit-able level:
;-------------------------------------------------------------------------------
Level:
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
 .db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$00,$FF,$00

;-------------------------------------------------------------------------------
;Bitmapdata:
;-------------------------------------------------------------------------------
WhiteBall:
 .db $FF,$FF,$FF,$FF,$1,$FF,$FC,$0,$7F,$F8,$0,$3F,$F0,$0,$1F,$E0
 .db $0,$F,$C0,$0,$7,$C0,$0,$7,$80,$0,$3,$80,$0,$3,$80,$0
 .db $3,$80,$0,$3,$80,$0,$3,$80,$0,$3,$80,$0,$3,$C0,$0,$7
 .db $C0,$0,$7,$E0,$0,$F,$F0,$0,$1F,$F8,$0,$3F,$FC,$0,$7F,$FF
 .db $1,$FF,$FF,$FF,$FF,$FF,$FF,$FF

BlackBall1:
 .db $0,$FE,$0,$3,$1,$80,$4,$0,$40,$8,$0,$20,$10,$0,$10,$20
 .db $0,$8,$40,$0,$4,$40,$0,$4,$80,$0,$2,$80,$0,$2,$80,$0
 .db $2,$80,$0,$2,$80,$0,$2,$80,$0,$2,$A0,$0,$A,$50,$0,$14
 .db $6A,$0,$AC,$35,$55,$58,$1A,$AA,$B0,$D,$55,$60,$6,$AA,$C0,$3
 .db $55,$80,$0,$FE,$0,$0,$0,$0

BlackBall2:
 .db $0,$FE,$0,$3,$1,$80,$4,$0,$40,$8,$0,$20,$10,$0,$10,$20
 .db $0,$8,$40,$AA,$4,$45,$55,$44,$AA,$AA,$AA,$D5,$55,$56,$AA,$AA
 .db $AA,$D5,$55,$56,$AA,$AA,$AA,$D5,$55,$56,$AA,$AA,$AA,$45,$55,$44
 .db $40,$AA,$4,$20,$0,$8,$10,$0,$10,$8,$0,$20,$4,$0,$40,$3
 .db $1,$80,$0,$FE,$0,$0,$0,$0

BlackBall3:
 .db $0,$FE,$0,$3,$55,$80,$6,$AA,$C0,$D,$55,$60,$1A,$AA,$B0,$35
 .db $55,$58,$6A,$0,$AC,$50,$0,$14,$A0,$0,$A,$80,$0,$2,$80,$0
 .db $2,$80,$0,$2,$80,$0,$2,$80,$0,$2,$80,$0,$2,$40,$0,$4
 .db $40,$0,$4,$20,$0,$8,$10,$0,$10,$8,$0,$20,$4,$0,$40,$3
 .db $1,$80,$0,$FE,$0,$0,$0,$0

;-------------------------------------------------------------------------------
;This contains savegame
;-------------------------------------------------------------------------------
LevelSaved: .db 0  ;standard: no level saved
SavedLevel: .db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

;--- ZCP encoder - Stephane JANTZEN - 12/95
;--- Beginning of ZCP data stream

TitlePicture:
 .db $00,$3F,$2C,$C0,$3F,$3F,$22,$C0,$0A,$C0,$3F,$A0,$3F,$2C,$C0,$80
 .db $E0,$22,$C0,$90,$84,$37,$E0,$FF,$39,$C0,$34,$4D,$19,$C0,$82,$81
 .db $0A,$E0,$1B,$E3,$80,$C0,$83,$F0,$98,$88,$10,$C0,$98,$98,$84,$BE
 .db $12,$C0,$BE,$FC,$10,$E2,$87,$B0,$12,$E0,$BC,$F0,$83,$E6,$18,$F6
 .db $EF,$82,$09,$E9,$E3,$C0,$13,$FB,$48,$0B,$E1,$E0,$09,$C0,$B9,$F0
 .db $F0,$A4,$8D,$9F,$98,$18,$FE,$8F,$0B,$E9,$FC,$C0,$09,$F0,$F0,$98
 .db $BC,$ED,$9F,$CC,$19,$F0,$8E,$84,$8D,$DC,$0B,$C0,$F0,$B0,$DC,$8F
 .db $BE,$9F,$E7,$1A,$E0,$87,$91,$EE,$B0,$0A,$E0,$F0,$88,$EC,$87,$FF
 .db $8F,$F3,$C0,$14,$E3,$E3,$E3,$4A,$83,$C1,$C1,$E8,$C5,$DC,$86,$F7
 .db $8F,$F3,$F0,$13,$E3,$F9,$F1,$DF,$F0,$FF,$9C,$87,$B1,$E1,$CC,$F7
 .db $B8,$BE,$B9,$C0,$10,$E3,$F9,$E3,$8F,$F9,$C3,$F8,$83,$CF,$D9,$DE
 .db $E3,$B9,$80,$F0,$90,$10,$E3,$F9,$C3,$8F,$CF,$80,$F8,$81,$FD,$CF
 .db $FB,$E2,$98,$E3,$E0,$17,$E1,$F3,$C3,$8F,$8F,$09,$F0,$87,$E6,$BF
 .db $CF,$08,$C7,$FC,$1B,$E0,$87,$C7,$8F,$8F,$C0,$8E,$83,$F9,$CF,$E1
 .db $C0,$88,$BE,$1F,$E8,$BF,$DC,$8E,$9F,$90,$8C,$87,$F3,$9F,$E1,$09
 .db $CB,$23,$E7,$4C,$83,$C7,$EE,$83,$C3,$9C,$E7,$F8,$A0,$84,$8C,$20
 .db $E3,$4C,$93,$C7,$E7,$C1,$C6,$8C,$E3,$F8,$A0,$82,$AC,$1F,$F0,$FF
 .db $BC,$F3,$8F,$CF,$E3,$D8,$98,$E3,$F2,$D0,$C4,$D8,$1D,$FC,$BF,$8E
 .db $B9,$C7,$E7,$F1,$F8,$8D,$B0,$B0,$A6,$C0,$B7,$1D,$E6,$8F,$CF,$9C
 .db $EB,$F3,$F8,$FE,$9E,$CC,$80,$D1,$E8,$8B,$F0,$16,$C3,$A3,$E7,$08
 .db $E3,$E3,$F8,$4C,$8B,$08,$C8,$FA,$A6,$EC,$15,$C0,$B4,$B1,$E1,$C5
 .db $BD,$BF,$9F,$FF,$8D,$C8,$B6,$9E,$F1,$D8,$A0,$18,$EC,$A3,$C0,$CA
 .db $F0,$FC,$B8,$F8,$B7,$CF,$C8,$BD,$D3,$B8,$1E,$F6,$97,$87,$8C,$F2
 .db $FC,$BA,$0A,$FE,$B9,$C7,$B3,$B9,$C0,$1A,$FA,$93,$F3,$E2,$B3,$BC
 .db $9B,$80,$FE,$BC,$9E,$B1,$F8,$BC,$1E,$FC,$E4,$8B,$E3,$F4,$F9,$89
 .db $C7,$F8,$9F,$F7,$F0,$EF,$F8,$1D,$E6,$F9,$E9,$F1,$FD,$FD,$09,$49
 .db $83,$FE,$9F,$80,$FC,$20,$E3,$FF,$88,$FC,$BF,$EE,$E0,$49,$B0,$B1
 .db $E0,$E0,$86,$22,$C1,$F3,$4A,$B1,$EC,$99,$D7,$F8,$98,$C0,$E0,$E0
 .db $84,$29,$E0,$49,$9F,$E0,$8E,$87,$E0,$84,$09,$C0,$C0,$98,$31,$FF
 .db $8F,$C0,$9F,$FF,$18,$C1,$08,$C0,$32,$F0,$0D,$4A,$0E,$C0,$14,$C0
 .db $33,$E0,$0B,$E1,$E0,$19,$C4,$3F,$98,$0A,$C0,$90,$27,$C0,$21,$C0
 .db $0E,$C0,$10,$C0,$18,$C0,$3F,$3C,$E0,$14,$C0,$13,$E0,$0D,$C0,$0D
 .db $C0,$1E,$E0,$2E,$E0,$11,$C0,$08,$C0,$1E,$E0,$0F,$C0,$18,$FC,$10
 .db $E0,$08,$F0,$1E,$FF,$0E,$C0,$1D,$ED,$C0,$87,$0E,$E0,$1E,$E2,$0B
 .db $E0,$1B,$E6,$EC,$80,$EC,$FC,$BE,$B6,$BC,$9E,$F8,$81,$C0,$F8,$F8
 .db $1E,$FC,$BC,$81,$CC,$EE,$E6,$BC,$E6,$86,$E0,$81,$F8,$E1,$E0,$1E
 .db $E6,$98,$83,$FC,$E6,$E6,$B9,$FE,$FE,$FC,$81,$E0,$FC,$FC,$1E,$E6
 .db $98,$87,$8C,$E6,$E6,$B1,$E0,$E6,$8C,$81,$E0,$8C,$8C,$1F,$FC,$B0
 .db $8C,$9D,$CD,$FC,$E3,$FD,$FC,$98,$83,$FD,$FB,$F8,$25,$E0,$0A,$E0
 .db $21,$E0,$09,$E0,$2F,$C0,$0D,$C0,$1E,$E0,$3E,$C0,$11,$C0,$19,$F0
 .db $3F,$38,$C0,$3F,$3F,$3F,$3F,$3F,$3C,$E0,$18,$F0,$08,$C0,$24,$C8
 .db $C0,$A0,$99,$0B,$E0,$18,$D0,$08,$C0,$20,$C1,$F4,$F3,$C2,$CE,$A8
 .db $90,$A0,$99,$C6,$F5,$08,$F4,$F0,$1C,$F2,$A2,$A9,$AA,$84,$D4,$8B
 .db $D5,$EA,$D5,$A3,$9C,$D5,$94,$1F,$C4,$C4,$F3,$C4,$89,$B8,$95,$BA
 .db $D5,$AE,$F5,$A9,$AA,$B8,$1C,$D1,$95,$90,$C5,$82,$A2,$85,$AA,$F5
 .db $AA,$85,$AE,$AE,$CA,$1E,$F3,$9D,$D8,$C5,$F3,$CC,$BD,$EA,$D5,$AB
 .db $DD,$AA,$EA,$EA,$80,$00

.end
