;****************************************************************************
;                             Sprite Routine Library
;                      optimized by Austin Butler for Usgard
;;****************************************************************************
#include "usgard.h"
#include "spritlib.h"

.org 0

.db 11

.dw fPUTSPRIT
.dw fNASRWARP
.dw fSET_DEST

fPUTSPRIT: ; Puts a sprite stored at (HL) at B,C
      push bc
      push de
graphmem:
      ld de,VIDEO_MEM
convdone:
      ld a,63
      sub c
      ld c,a
      push hl
      push hl
      call FIND_PIXEL
      add hl,de
      ex de,hl
      pop hl
      ld b,(hl)
      inc hl
      ld c,(hl)
      inc hl
      push hl
      pop ix
      ex de,hl
PS_NewRow:
      push bc
      ld d,(ix)
      inc ix
      push af
      push hl
PS_NewCol:
      rl d
      ld e,a
      jr nc,PS_NoPixel
      or (hl)
      ld (hl),a
      jr PS_NextPixel
PS_NoPixel:
      cpl
      and (hl)
      ld (hl),a
PS_NextPixel:
      ld a,e
      rrca
      jr nc,PS_SameByte
      inc hl
PS_SameByte:
      djnz PS_NewCol
      pop hl
      pop af
      ld de,16
      add hl,de
      pop bc
      dec c
      jr nz,PS_NewRow
      pop hl
      pop de
      pop bc
      ret

fNASRWARP:
R_nw1:  ld      (NASR_Status),a
	push    bc
	ex      de,hl
	call    FIND_PIXEL
graphmem_n:     ld      bc, $fc00
	add     hl,bc
R_nw:   ld      (PicCoor),hl
	ex      de,hl
	ld      b,$ff
Patch:
	rla
	inc     b
	jr      nc,Patch
	ld      c,(hl)
	inc     hl
	ld      a,(hl)
R_p:    ld      (offset),a
	ld      a,b
	ld      e,c
	cp      (hl)
	jr      c,DS1
	inc     e
DS1:
	inc     hl
	ld      d,(hl)
	inc     hl
YLoop:
	push    bc
	push    de
        ld      b,16
	ex      de,hl
R_yl:   ld      hl,TempLine
InitTempLineLoop:
	ld      (hl),$ff
	inc     hl
	djnz    InitTempLineLoop
	ex      de,hl
R_itl:  ld      a,(NASR_Status)
	and     a
	jr      z,NASRErase
NASRDraw:
	ld      b,0
R_ND:   ld      de,TempLine
	ldir
	jr      NASRDrawFin
NASRErase:
	ld      b,c
R_NE:   ld      hl,TempLine
LoadEmpty:
	ld      (hl),0
	inc     hl
	djnz    LoadEmpty
NASRDrawFin:
	pop     de
	pop     bc
	push    hl
	push    bc
	push    de
	ld      d,b
	ld      a,b
	and     a
	jr      z,skipshift
SpriLoop1:
	ld      a,b
R_SL:   ld      HL,TempLine
	ld      b,e
	scf
SpriLoop2:
	rr      (HL)
	inc     HL
	djnz    SpriLoop2
	ld      b,a
	djnz    SpriLoop1
	ld      b,d
skipshift:
	ld      a,$ff           ;fill accumulator
	inc     b
mask1:
	srl     a               ;make the mask
	djnz    mask1           ;something like
	scf                     ;00001111 ->
	rla
R_m1:   ld      hl,(PicCoor)    ;implement the mask
	or      (hl)            ;this preserve the 0 bits
R_m2:   ld      hl,TempLine   ;become xxxx1111
	and     (hl)            ;when anded, become
	ld      (hl),a          ;xxxxyyyy
	ld      b,d             ;retrieve b
R_m3:   ld      a,(offset)
	dec     a
	sub     b
	jr      nc,skip
	sub     248
skip:
	ld      b,a
	inc     b
	ld      a,$ff
mask2:
	add     a, a
	djnz    mask2
	scf
	rra
	dec     e
R_m4:   ld      hl,(PicCoor)
	ld      d,0
	add     hl,de
	or      (hl)
R_m5:   ld      hl,TempLine
	add     hl,de
	and     (hl)
	ld      (hl),a
	inc     e
	ld      c,e
R_m6:   ld      hl,(PicCoor)
	ld      de,16
	push    hl
	add     hl,de
R_m7:   ld      (PicCoor),hl
	pop     hl
R_m8:   ld      de,TempLine
	ex      de,hl
	ld      b,0
	ldir
	pop     de
	pop     bc
	pop     hl
	dec     d
R_lb    jp      nz,YLoop
	pop     bc
	ret

fSET_DEST:
	ld      (&graphmem+1), hl
	ld      (&graphmem_n+1), hl
	ret

TempLine:
.dw 0,0,0,0,0,0,0,0
PicCoor:
.dw 0
offset:
.dw 0
NASR_Status:
.db 0

.end
